﻿using Infragistics.Web.UI.GridControls;
using System;
using System.Collections.Generic;
using System.Data;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace WHDGDynamicColumns
{
    public partial class WebForm1 : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (Session["data"] == null)
            {
                Session["data"] = CreateDataSource();
            }

            if (!Page.IsPostBack)
            {
                this.WebHierarchicalDataGrid1.GridView.Behaviors.CreateBehavior<SummaryRow>();
                this.WebHierarchicalDataGrid1.GridView.Behaviors.SummaryRow.EnableInheritance = true;
                AddCustomSummary();
            }

            this.WebHierarchicalDataGrid1.CalculateCustomSummary += WebDataGrid1_CalculateCustomSummary;
            this.WebHierarchicalDataGrid1.RowIslandCreated += WebHierarchicalDataGrid1_RowIslandCreated;
            this.WebHierarchicalDataGrid1.RowIslandDataBinding += WebHierarchicalDataGrid1_RowIslandDataBinding;

            WebHierarchicalDataGrid1.DataSource = Session["data"];
            WebHierarchicalDataGrid1.DataBind();
        }

        private void WebHierarchicalDataGrid1_RowIslandDataBinding(object sender, RowIslandEventArgs e)
        {
            e.RowIsland.CalculateCustomSummary += WebDataGrid1_CalculateCustomSummary;
        }

        public void WebHierarchicalDataGrid1_RowIslandCreated(object sender, RowIslandEventArgs e)
        {
            AddChildRowIslandCustomSummary(e.RowIsland);
        }

        protected void AddChildRowIslandCustomSummary(ContainerGrid ri)
        {
            Summary customSummary = new Summary();
            customSummary.SummaryType = SummaryType.Custom;
            customSummary.CustomSummaryName = "Custom2";

            ColumnSummaryInfo customInfo = new ColumnSummaryInfo();
            customInfo.ColumnKey = "ItemCh";
            customInfo.Summaries.Add(customSummary);
            ri.Behaviors.SummaryRow.ColumnSummaries.Add(customInfo);

            if (ri.Behaviors.SummaryRow.ColumnSettings["ItemCh"] == null)
            {
                SummaryRowSetting summaryRowSetting = new SummaryRowSetting(ri, "ItemCh");
                SummarySetting customSummarySetting = new SummarySetting();
                customSummarySetting.SummaryType = SummaryType.Custom;
                customSummarySetting.CustomSummaryName = "Custom2";
                customSummarySetting.FormatString = "Custom2 = {1}";
                summaryRowSetting.ColumnKey = "ItemCh";
                summaryRowSetting.SummarySettings.Add(customSummarySetting);
                ri.Behaviors.SummaryRow.ColumnSettings.Add(summaryRowSetting);
            }
        }

        public object WebDataGrid1_CalculateCustomSummary(object sender, CustomSummaryEventArgs e)
        {
            if(e.Summary.CustomSummaryName == "Custom")
            {
                return "Custom Result";
            }
            return "other";
        }

        public void AddCustomSummary()
        {
            Summary customSummary = new Summary();
            customSummary.SummaryType = SummaryType.Custom;
            customSummary.CustomSummaryName = "Custom";

            ColumnSummaryInfo customInfo = new ColumnSummaryInfo();
            customInfo.ColumnKey = "Item";
            customInfo.Summaries.Add(customSummary);
            WebHierarchicalDataGrid1.GridView.Behaviors.SummaryRow.ColumnSummaries.Add(customInfo);

            SummaryRowSetting summaryRowSetting = new SummaryRowSetting(this.WebHierarchicalDataGrid1.GridView, "Item");
            SummarySetting customSummarySetting = new SummarySetting();
            customSummarySetting.SummaryType = SummaryType.Custom;
            customSummarySetting.CustomSummaryName = "Custom";
            customSummarySetting.FormatString = "Custom = {1}";
            summaryRowSetting.ColumnKey = "Item";
            summaryRowSetting.SummarySettings.Add(customSummarySetting);

            if (WebHierarchicalDataGrid1.GridView.Behaviors.SummaryRow.ColumnSettings["Item"] == null)
            {
                WebHierarchicalDataGrid1.GridView.Behaviors.SummaryRow.ColumnSettings.Add(summaryRowSetting);
            }
        }

        private DataSet CreateDataSource()
        {
            DataSet ds = new DataSet();
            ds.Tables.Add(CreateGridDataSourceForModules("Module"));
            ds.Tables.Add(CreateGridDataSourceForSubModules("TRN_CODE", true));
            DataRelation dr = new DataRelation("id", ds.Tables["Module"].Columns["id"], ds.Tables["TRN_CODE"].Columns["ParentId"], false);
            ds.Relations.Add(dr);
            return ds;
        }

        private DataTable CreateGridDataSourceForModules(string tableName)
        {
            System.Data.DataTable table = new DataTable(tableName);
            DataColumn column;
            DataRow row;
            column = new DataColumn();
            column.DataType = System.Type.GetType("System.Int32");
            column.ColumnName = "id";
            column.ReadOnly = true;
            table.Columns.Add(column);
            column = new DataColumn();
            column.DataType = System.Type.GetType("System.Int32");
            column.ColumnName = "Item";
            column.AutoIncrement = false;
            column.Caption = "Item";
            column.ReadOnly = false;
            column.Unique = false;
            table.Columns.Add(column);
            DataColumn[] PrimaryKeyColumns = new DataColumn[1];
            PrimaryKeyColumns[0] = table.Columns["id"];
            table.PrimaryKey = PrimaryKeyColumns;

            for (int i = 0; i <= 10 - 1; i++)
            {
                row = table.NewRow();
                row["id"] = i;
                row["Item"] = i;
                table.Rows.Add(row);
            }

            return table;
        }

        private DataTable CreateGridDataSourceForSubModules(string tableName, bool duplicate)
        {
            System.Data.DataTable table = new DataTable(tableName);
            DataColumn column;
            DataRow row;
            column = new DataColumn();
            column.DataType = System.Type.GetType("System.Int32");
            column.ColumnName = "ChildId";
            column.AutoIncrement = false;
            column.Caption = "ChildId";
            column.ReadOnly = false;
            column.Unique = false;
            table.Columns.Add(column);
            column = new DataColumn();
            column.DataType = System.Type.GetType("System.Int32");
            column.ColumnName = "ItemCh";
            column.AutoIncrement = false;
            column.Caption = "ItemCh";
            column.ReadOnly = false;
            column.Unique = false;
            table.Columns.Add(column);
            column = new DataColumn();
            column.DataType = System.Type.GetType("System.Int32");
            column.ColumnName = "ParentId";
            column.ReadOnly = true;
            table.Columns.Add(column);
            column = new DataColumn();
            column.DataType = System.Type.GetType("System.Boolean");
            column.ColumnName = "ChView";
            column.AutoIncrement = false;
            column.Caption = "View";
            column.Unique = false;
            column.DefaultValue = false;
            table.Columns.Add(column);
            column = new DataColumn();
            column.DataType = System.Type.GetType("System.Boolean");
            column.ColumnName = "ChUpdate";
            column.AutoIncrement = false;
            column.ReadOnly = false;
            column.Unique = false;
            column.DefaultValue = false;
            table.Columns.Add(column);
            column = new DataColumn();
            column.DataType = System.Type.GetType("System.Boolean");
            column.ColumnName = "ChDelete";
            column.AutoIncrement = false;
            column.Caption = "Delete";
            column.Unique = false;
            column.DefaultValue = false;
            table.Columns.Add(column);
            column = new DataColumn();
            column.DataType = System.Type.GetType("System.Boolean");
            column.ColumnName = "ChSave";
            column.AutoIncrement = false;
            column.Caption = "Save";
            column.ReadOnly = false;
            column.Unique = false;
            column.DefaultValue = false;
            table.Columns.Add(column);

            if (!duplicate)
            {
                DataColumn[] PrimaryKeyColumns = new DataColumn[1];
                PrimaryKeyColumns[0] = table.Columns["id"];
                table.PrimaryKey = PrimaryKeyColumns;
            }
            else
            {
                DataColumn[] PrimaryKeyColumns = new DataColumn[1];
                PrimaryKeyColumns[0] = table.Columns["ChildId"];
                table.PrimaryKey = PrimaryKeyColumns;
            }

            for (int i = 0; i <= 5 - 1; i++)
            {
                row = table.NewRow();
                row["ParentId"] = i;
                row["ItemCh"] = i*10;
                row["ChildId"] = i + 10;
                table.Rows.Add(row);
            }

            return table;
        }
    }
}